/***********************************************************************
*
* Copyright (c) 2021-2022 Barbara Geller and Ansel Sermersheim
*
* This example is free software, released under the BSD 2-Clause license.
* For license details refer to LICENSE provided with this project.
*
* CopperSpice is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*
* https://opensource.org/licenses/BSD-2-Clause
*
***********************************************************************/

#include <QtCore>
#include <QtGui>

class MainWindow : public QWidget
{
 public:
   MainWindow();

 private:
   QByteArray readJson(QString fName);
};

MainWindow::MainWindow()
{
   setMinimumSize(600, 400);

   // example 1
   QPushButton *plus_PB  = new QPushButton("plus");
   QPushButton *minus_PB = new QPushButton("minus");

   QFrame *frame = new QFrame();

   QPixmap plusImage(":/resources/plus.png");
   plus_PB->setIcon(plusImage);

   QPixmap minusImage(":/resources/minus.png");
   minus_PB->setIcon(minusImage);

   // example 2
   QComboBox *wordList_CB = new QComboBox();
   wordList_CB->setMinimumWidth(250);

   QString fileName = ":/resources/words.txt";

   QFile dictionary(fileName);
   dictionary.open(QFile::ReadOnly);

   QTextStream inputStream(&dictionary);

   while (! inputStream.atEnd()) {
      wordList_CB->addItem(inputStream.readLine());
   }

   wordList_CB->model()->sort(0);
   wordList_CB->setCurrentIndex(0);

   // example 3
   QByteArray data = readJson(":/resources/config.json");

   QJsonDocument doc  = QJsonDocument::fromJson(data);
   QJsonObject object = doc.object();
   QVariantMap map    = object.toVariantMap();

   QTableView *view   = new QTableView();

   QStandardItemModel *model = new QStandardItemModel(map.size(), 2);
   model->setHeaderData(0, Qt::Horizontal, QString("Json Key"));
   model->setHeaderData(1, Qt::Horizontal, QString("Value"));

   view->setModel(model);
   view->setColumnWidth(0, 200);
   view->setColumnWidth(1, 250);
   view->horizontalHeader()->setStretchLastSection(true);

   int row = 0;
   QStandardItem *item;

   for (auto iter = map.begin(); iter != map.end(); ++iter) {
      item = new QStandardItem(iter.key());
      model->setItem(row, 0, item);

      item = new QStandardItem(iter.value().toString());
      model->setItem(row, 1, item);

      ++row;
   }

   QHBoxLayout *layout1 = new QHBoxLayout();
   layout1->addStretch();
   layout1->addWidget(plus_PB);
   layout1->addSpacing(20);
   layout1->addWidget(minus_PB);
   layout1->addStretch();

   QPushButton *close_PB = new QPushButton();
   close_PB->setText("Close");

   QHBoxLayout *layout2 = new QHBoxLayout();
   layout2->addStretch();
   layout2->addWidget(close_PB);
   layout2->addStretch();

   QVBoxLayout *layoutMain = new QVBoxLayout(this);
   layoutMain->setContentsMargins(45, 35, 45, 25);

   layoutMain->addLayout(layout1);
   layoutMain->addSpacing(30);
   layoutMain->addWidget(wordList_CB, 0, Qt::AlignHCenter);
   layoutMain->addSpacing(30);
   layoutMain->addWidget(view);
   layoutMain->addSpacing(20);
   layoutMain->addLayout(layout2);

   connect(close_PB, &QPushButton::clicked, this, &QWidget::close);
}

QByteArray MainWindow::readJson(QString fName)
{
   QByteArray data;

   QFile file(fName);
   if (! file.open(QFile::ReadOnly | QFile::Text)) {
      const QString msg = "Unable to open Syntax file: " + fName + " : " + file.errorString();
      QMessageBox::critical(nullptr, "Open File", msg, QMessageBox::Cancel);

      return data;
   }

   data = file.readAll();
   file.close();

   return data;
}

int main(int argc, char *argv[])
{
   QApplication app(argc, argv);

   MainWindow *window = new MainWindow();
   window->show();

   return app.exec();
}